@@ -1,12 +1,13 @@
 # PyQt5 Video player
 #!/usr/bin/env python
 
+from libs.zoomWidget import ZoomWidget
 import argparse
 import logging
 from datetime import datetime, timedelta
 import time
 
-__version_info__ = ('0', '0', '14')
+__version_info__ = ('0', '0', '15')
 __version__ = '.'.join(__version_info__)
 
 from src.VideoTrackingWindow import Ui_VideoTrackingWindow
@@ -93,6 +94,7 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         self.statusBar.addPermanentWidget(self.statusProgressBar, 1)
         self.statusBar.addPermanentWidget(self.labelCoordinates, 2)
 
+        self.zoomWidget = ZoomWidget()
         self.canvas = Canvas(parent=self)
         self.scrollArea.setWidget(self.canvas)
         self.canvas.scrollRequest.connect(self.scrollRequest)
@@ -101,6 +103,8 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         self.canvas.selectionChanged.connect(self.shapeSelectionChanged)
 
         # self.canvas.zoomRequest.connect(self.zoomNavig.zoomRequest)
+        self.canvas.zoomRequest.connect(self.zoomRequest)
+
         self.previous_shape = []
 
         self.videolen = 0
@@ -165,6 +169,68 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         self.playButton.setIcon(
             self.style().standardIcon(QStyle.SP_MediaPause))
 
+    def zoomRequest(self, delta):
+        # get the current scrollbar positions
+        # calculate the percentages ~ coordinates
+        h_bar = self.scrollBars[Qt.Horizontal]
+        v_bar = self.scrollBars[Qt.Vertical]
+
+        # get the current maximum, to know the difference after zooming
+        h_bar_max = h_bar.maximum()
+        v_bar_max = v_bar.maximum()
+
+        # get the cursor position and canvas size
+        # calculate the desired movement from 0 to 1
+        # where 0 = move left
+        #       1 = move right
+        # up and down analogous
+        cursor = QCursor()
+        pos = cursor.pos()
+        relative_pos = QWidget.mapFromGlobal(self, pos)
+
+        cursor_x = relative_pos.x()
+        cursor_y = relative_pos.y()
+
+        w = self.scrollArea.width()
+        h = self.scrollArea.height()
+
+        # the scaling from 0 to 1 has some padding
+        # you don't have to hit the very leftmost pixel for a maximum-left movement
+        margin = 0.1
+        move_x = (cursor_x - margin * w) / (w - 2 * margin * w)
+        move_y = (cursor_y - margin * h) / (h - 2 * margin * h)
+
+        # clamp the values from 0 to 1
+        move_x = min(max(move_x, 0), 1)
+        move_y = min(max(move_y, 0), 1)
+
+        # zoom in
+        units = delta / (8 * 15)
+        scale = 10
+        self.addZoom(scale * units)
+
+        # get the difference in scrollbar values
+        # this is how far we can move
+        d_h_bar_max = h_bar.maximum() - h_bar_max
+        d_v_bar_max = v_bar.maximum() - v_bar_max
+
+        # get the new scrollbar values
+        new_h_bar_value = h_bar.value() + move_x * d_h_bar_max
+        new_v_bar_value = v_bar.value() + move_y * d_v_bar_max
+
+        h_bar.setValue(new_h_bar_value)
+        v_bar.setValue(new_v_bar_value)
+
+
+    def addZoom(self, increment=10):
+        self.setZoom(self.zoomWidget.value() + increment)
+
+    def setZoom(self, value):
+        # self.actions.fitWidth.setChecked(False)
+        # self.actions.fitWindow.setChecked(False)
+        # self.zoomMode = self.MANUAL_ZOOM
+        self.zoomWidget.setValue(value)
+
     def setPosition(self, index):
         if index < 0:
              index = 0
@@ -231,6 +297,7 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
             # read number frames
             videogen = skvideo.io.FFmpegReader(self.filePath)
             (numFiles, width, height, _) = videogen.getShape()
+            videogen = skvideo.io.vreader(self.filePath)
 
             self.width = int(1920/2)
             self.height = int(1440/2)
